#!/usr/bin/env python3
import re
import sys
from os.path import basename
from pathlib import Path
from typing import List, NamedTuple


class BlogEntry(NamedTuple):
    path: Path
    title: str
    date: str

    @staticmethod
    def from_path(path: Path):
        try:
            with open(path, "r") as file:
                content = file.read()
                title = re.search(r"(?<=<h1>).*?(?=</h1>)", content)
                if title is None:
                    raise Exception("No title found")
                date = re.search(r"(?<=<small>).*?(?=</small>)", content)
                if date is None:
                    raise Exception("No date found")
                return BlogEntry(path, title.group(0), date.group(0))
        except Exception as e:
            print("ERROR: " + str(e))
            return None


def process_file(url: str, entry: BlogEntry, last: bool) -> None:
    print(
        f"<a class='blog-entry' href='/{url}/{basename(entry.path).removesuffix(".html")}'>{entry.title}</a>"
    )
    print("<br>")
    print(f"Posted at {entry.date}")
    if not last:
        print("<hr>")


def get_blog_entries(directory: Path) -> List[BlogEntry | None]:
    return [BlogEntry.from_path(file) for file in directory.rglob("*.html")]


def process_files(directory: Path) -> None:
    entries = [e for e in get_blog_entries(directory) if e is not None]
    entries.sort(reverse=True, key=lambda entry: (entry.date, entry.title))
    for i, entry in enumerate(entries):
        process_file(basename(directory), entry, i == len(entries) - 1)


if __name__ == "__main__":
    directory = Path(sys.argv[1])
    process_files(directory)
